#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

import sys
from setuptools import setup, find_packages


setup(
    name='li_rn_networks',
    version='0.1',
    description='neural networks. It have recurrent connections and leaky integrator neuron.',
    long_description_content_type='text/markdown',
    author='picker.',
    author_email='picker@pickerlav.jp',
    url='https://github.com/shibuiwilliam/py-kaomoji.git',
    packages=['li_rn_networks'],
    include_package_data=True,
)
